# =========================== #
#
# Jenkins et al. 2020 Ecology and Evolution
# R script to plot map
# 
# Description:
# Plot a map and add an image to the map.
#
# Notes before execution:
# 1. Make sure all required R packages are installed.
# 2. Set working directory to the location of this R script.
#
# =========================== #

# Load libraries
library(raster)
library(rworldmap)
library(rworldxtra)
library(dplyr)
library(ggplot2)
library(ggsn)
library(jpeg)

# Set map boundary (xmin, xmax, ymin, ymax)
boundary = extent(-12, 20, 35, 55)
boundary

# Download a high resolution basemap
map.outline = getMap(resolution = "high")

# Crop to boundary and convert to dataframe
map.outline = crop(map.outline, y = boundary) %>% fortify()

# Import coordinates
coords = read.csv("Data/Corsica_coordinates.csv")
coords

# Plot basemap
basemap = ggplot()+
  geom_polygon(data=map.outline, aes(x=long, y=lat, group=group), fill="grey",
               colour="black", size=0.5)+
  coord_quickmap(expand = F)+
  ggsn::north(map.outline, symbol = 10, scale = 0.06, location = "bottomright",
              anchor = c(x = 19, y = 36))+
  ggsn::scalebar(data = map.outline, dist = 200, dist_unit = "km", height = 0.01,
                 transform = TRUE, model = "WGS84", 
                 location = "bottomright", anchor = c(x = 19, y = 35),
                 st.bottom = FALSE, st.size = 4, st.dist = 0.015, border.size = 0.5)+
  xlab("Longitude")+
  ylab("Latitude")+
  theme(
    axis.text = element_text(colour="black", size=12),
    axis.title = element_text(colour="black", size=14),
    panel.background = element_rect(fill="lightsteelblue2"),
    panel.border = element_rect(fill = NA, colour = "black", size = 0.5),
    panel.grid.minor = element_line(colour="grey90", size=0.5),
    panel.grid.major = element_line(colour="grey90", size=0.5),
    plot.margin = margin(0, 0.5, 0, 0.5, "cm")
  )
# basemap

# Add points and text to basemap
map1 = basemap + 
  # Add point data
  geom_point(data=coords, aes(x=Lon, y=Lat, fill=Sea), size=5, shape=21,
             colour="black", show.legend=FALSE)+
  # BASTIA
  geom_label(aes(x=10.8, y=43.2), label="Bastia", size=5, colour="black", 
             fontface="bold", label.padding = unit(0.30, "lines"))+
  # CALVI
  geom_label(aes(x=7.5, y=43), label="Calvi", size=5, colour="black", 
             fontface="bold", label.padding = unit(0.30, "lines"))+
  # ILE D'YEU
  geom_label(aes(x=0.5, y=47.25),label=paste(coords$Sea[1],"hatchery",sep=" "),
             size=5, colour="black",
             fontface="bold", label.padding = unit(0.30, "lines"))+
  # Other text
  geom_text(aes(x=9.2, y=42), label="Corsica", size=4, fontface="bold")+
  geom_text(aes(x=9.2, y=40), label="Sardinia", size=4, fontface="bold")+
  geom_text(aes(x=7.5, y=38.2), label="Mediterranean", size=6, fontface="bold")+
  geom_text(aes(x=-7, y=47), label="Atlantic Ocean", size=6, fontface="bold")+
  geom_text(aes(x=-3.8, y=45.5), label="Bay of Biscay", size=4, fontface="bold")+
  scale_fill_manual(values=c("white","white"), labels=c(" Corsica ", " ?le d'Yeu "))
# map1

# Prepare lobster image
jpg = readJPEG("./Data/Alex_Hyde_S5_larva.jpg")
raster_img = rasterGrob(jpg, interpolate=TRUE)
image = ggplot() +
  annotation_custom(raster_img, xmin=-Inf, xmax=Inf, ymin=-Inf, ymax=Inf)+
  theme(
    panel.background = element_blank(),
    plot.background = element_blank(),
    panel.border = element_blank()
  )
# image

# Add lobster image to map
map1 +
  annotation_custom(
    grob = ggplotGrob(image),
    xmin=7,
    xmax=Inf,
    ymin=44.5,
    ymax=Inf) + 
  geom_text(aes(x=9.9, y=47.5), label="Stage V larva", size=5, fontface="bold",
            color="white") +
  geom_segment(aes(x=7.9, y=47, xend=10.2, yend=47), colour="white")+
  geom_text(aes(x=7.9, y=46.75), label="0", size=3, color="white") +
  geom_text(aes(x=10.6, y=46.75), label="1 cm", size=3, color="white")
# ggsave("Fig2.png", width=10, height=9.5, dpi=600)
# ggsave("Fig2.tiff", width=10, height=9.5, dpi=600)
