# =========================== #
#
# Jenkins et al. 2020 Ecology and Evolution
# R script to analyse Corsica SNP genotypes
# 
# Description:
# Plot a map and add an image to the map.
#
# Notes before execution:
# 1. Make sure all required R packages are installed.
# 2. Set working directory to the location of this R script.
#
# =========================== #

# Load libraries
library(adegenet)
library(poppr)
library(dplyr)
library(miscTools)


# ----------------- #
#
# Data import and preparation
#
# ----------------- #

# Import and convert raw data
rawdata = read.csv("Data/Lobster_SNP_Genotypes_Corsica.csv", header=TRUE)
rawdata = reshape(rawdata, idvar=c("Sample","Site"), timevar="Locus",
                  direction = "wide", sep="")

# Remove "Genotype" from column names
colnames(rawdata)
colnames(rawdata) = gsub("Genotype", "", colnames(rawdata)); colnames(rawdata)

# Subset genotype data
geno = rawdata[ , 3:98]

# Create vector of individuals and of population IDs
ind = as.character(rawdata$Sample) # individual labels
population = as.character(rawdata$Site) # population labels

# Remove all SNP loci that were omitted in Jenkins et al. 2019
# https://doi.org/10.1111/eva.12849
snps_to_remove = c("25580","32362","41521","53889","65376", # poor performance on Fluidigm EP1
                   "22365", # missing data
                   "8953","21197", # hwe
                   "15531","22740","28357","33066","51507","53052","53263", # LD
                   "21880","22323") # heterozygosity
geno = geno[ , !colnames(geno) %in% snps_to_remove]

# Convert to Genind object
snpdata = df2genind(geno, ploidy=2, ind.names=ind, pop=population, sep="")
snpdata
nLoc(snpdata) # number of loci
nPop(snpdata) # number of sites
nInd(snpdata) # number of individuals
summary(snpdata$pop) # sample size


# ----------------- #
#
# Missing data check
#
# ----------------- #

# Loci missing data
locmiss = propTyped(snpdata, by="loc")
locmiss
locmiss[which(locmiss < 0.8)] # more than 20% missing data

# Indivual missing data
indmiss = propTyped(snpdata, by="ind")
indmiss
indmiss[which(indmiss < 0.7)] # more than 30% missing data

# Remove loci with missing data
snpdata = missingno(snpdata, type="loci", cutoff=0.2)

# Remove samples with missing data
snpdata = missingno(snpdata, type="geno", cutoff=0.3)
snpdata

# Check for potential duplicated samples in Corsica
missingno(popsub(snpdata, sublist = "Csa"), cutoff = 0) %>% mlg()

# Change Corsica labels
indNames(snpdata) = gsub("A01-Cent-13", "Csa01", x = indNames(snpdata))
indNames(snpdata) = gsub("A02-Cent-13", "Csa02", x = indNames(snpdata))
indNames(snpdata) = gsub("V13.8802", "Csa03", x = indNames(snpdata))
indNames(snpdata) = gsub("V13.8804", "Csa04", x = indNames(snpdata))
indNames(snpdata) = gsub("C14_01", "Csa05", x = indNames(snpdata))
indNames(snpdata) = gsub("C14_02", "Csa06", x = indNames(snpdata))
indNames(snpdata) = gsub("F05-2015", "Csa07", x = indNames(snpdata))
indNames(snpdata) = gsub("F07-2015", "Csa08", x = indNames(snpdata))
indNames(snpdata) = gsub("F08-2015", "Csa09", x = indNames(snpdata))
indNames(snpdata) = gsub("F02-2016", "Csa10", x = indNames(snpdata))
indNames(snpdata) = gsub("F03-2016", "Csa11", x = indNames(snpdata))
indNames(snpdata) = gsub("F05-2016", "Csa12", x = indNames(snpdata))
indNames(snpdata) = gsub("F06-2016", "Csa13", x = indNames(snpdata))
indNames(snpdata) = gsub("F01-2017", "Csa14", x = indNames(snpdata))
indNames(snpdata) = gsub("F02-2017", "Csa15", x = indNames(snpdata))
indNames(snpdata) = gsub("F03-2017", "Csa16", x = indNames(snpdata))
indNames(snpdata) = gsub("F04-2017", "Csa17", x = indNames(snpdata))
indNames(snpdata) = gsub("M01-2018", "Csa18", x = indNames(snpdata))
indNames(snpdata) = gsub("F010-2018", "Csa19", x = indNames(snpdata))
indNames(snpdata) = gsub("F01-2020", "Csa20", x = indNames(snpdata))
indNames(snpdata) %>% tail(n = 20)


# ----------------- #
#
# Export SNP genotypes
#
# ----------------- #

# Export all filtered genotypes
save(snpdata, file = "./Data/all_filt_geno.RData")

# Subset Corsica individuals
csa = popsub(snpdata, sublist = "Csa")
csa

# Export Corsica filtered genotypes
save(csa, file = "./Data/corsica_filt_geno.RData")

# Export Corsica filtered genotypes in genepop format
source("./Data/TJ_genind2genepop_function.R")
genind2genepop(csa, file = "./Data/corsica_filt_geno.gen")
